@@ -9,6 +9,7 @@ import (
 	iov1 "github.com/openshift/api/operatoringress/v1"
 	logf "github.com/openshift/cluster-ingress-operator/pkg/log"
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
+	operatorcontroller "github.com/openshift/cluster-ingress-operator/pkg/operator/controller"
 	retryable "github.com/openshift/cluster-ingress-operator/pkg/util/retryableerror"
 	"github.com/openshift/cluster-ingress-operator/pkg/util/slice"
 
@@ -46,6 +47,7 @@ const (
 	IngressControllerDeploymentAvailableConditionType            = "DeploymentAvailable"
 	IngressControllerDeploymentReplicasMinAvailableConditionType = "DeploymentReplicasMinAvailable"
 	IngressControllerDeploymentReplicasAllAvailableConditionType = "DeploymentReplicasAllAvailable"
+	IngressControllerCanaryCheckSuccessConditionType             = "CanaryChecksSucceeding"
 )
 
 var (
@@ -65,7 +67,7 @@ var (
 // in the manager namespace.
 func New(mgr manager.Manager, config Config) (controller.Controller, error) {
 	reconciler := &reconciler{
-		Config:   config,
+		config:   config,
 		client:   mgr.GetClient(),
 		cache:    mgr.GetCache(),
 		recorder: mgr.GetEventRecorderFor(controllerName),
@@ -95,7 +97,7 @@ func New(mgr manager.Manager, config Config) (controller.Controller, error) {
 func (r *reconciler) ingressConfigToIngressController(o handler.MapObject) []reconcile.Request {
 	var requests []reconcile.Request
 	controllers := &operatorv1.IngressControllerList{}
-	if err := r.cache.List(context.Background(), controllers, client.InNamespace(r.Namespace)); err != nil {
+	if err := r.cache.List(context.Background(), controllers, client.InNamespace(r.config.Namespace)); err != nil {
 		log.Error(err, "failed to list ingresscontrollers for ingress", "related", o.Meta.GetSelfLink())
 		return requests
 	}
@@ -142,7 +144,7 @@ type Config struct {
 // reconciler handles the actual ingress reconciliation logic in response to
 // events.
 type reconciler struct {
-	Config
+	config Config
 
 	client   client.Client
 	cache    cache.Cache
@@ -228,6 +230,18 @@ func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, err
 		return reconcile.Result{Requeue: true}, nil
 	}
 
+	// TODO: remove this fix-up logic in 4.8
+	if ingress.Status.EndpointPublishingStrategy != nil && ingress.Status.EndpointPublishingStrategy.Type == operatorv1.LoadBalancerServiceStrategyType && ingress.Status.EndpointPublishingStrategy.LoadBalancer == nil {
+		log.Info("Setting default value for empty status.endpointPublishingStrategy.loadBalancer field", "ingresscontroller", ingress)
+		ingress.Status.EndpointPublishingStrategy.LoadBalancer = &operatorv1.LoadBalancerStrategy{
+			Scope: operatorv1.ExternalLoadBalancer,
+		}
+		if err := r.client.Status().Update(context.TODO(), ingress); err != nil {
+			return reconcile.Result{}, fmt.Errorf("failed to update status: %v", err)
+		}
+		return reconcile.Result{Requeue: true}, nil
+	}
+
 	// The ingresscontroller is safe to process, so ensure it.
 	if err := r.ensureIngressController(ingress, dnsConfig, infraConfig, ingressConfig, apiConfig, networkConfig); err != nil {
 		switch e := err.(type) {
@@ -258,14 +272,14 @@ func (r *reconciler) admit(current *operatorv1.IngressController, ingressConfig
 	if err := r.validate(updated); err != nil {
 		switch err := err.(type) {
 		case *admissionRejection:
-			updated.Status.Conditions = mergeConditions(updated.Status.Conditions, operatorv1.OperatorCondition{
+			updated.Status.Conditions = MergeConditions(updated.Status.Conditions, operatorv1.OperatorCondition{
 				Type:    IngressControllerAdmittedConditionType,
 				Status:  operatorv1.ConditionFalse,
 				Reason:  "Invalid",
 				Message: err.Reason,
 			})
 			updated.Status.ObservedGeneration = updated.Generation
-			if !ingressStatusesEqual(current.Status, updated.Status) {
+			if !IngressStatusesEqual(current.Status, updated.Status) {
 				if err := r.client.Status().Update(context.TODO(), updated); err != nil {
 					return fmt.Errorf("failed to update status: %v", err)
 				}
@@ -274,13 +288,13 @@ func (r *reconciler) admit(current *operatorv1.IngressController, ingressConfig
 		return err
 	}
 
-	updated.Status.Conditions = mergeConditions(updated.Status.Conditions, operatorv1.OperatorCondition{
+	updated.Status.Conditions = MergeConditions(updated.Status.Conditions, operatorv1.OperatorCondition{
 		Type:   IngressControllerAdmittedConditionType,
 		Status: operatorv1.ConditionTrue,
 		Reason: "Valid",
 	})
 	updated.Status.ObservedGeneration = updated.Generation
-	if !ingressStatusesEqual(current.Status, updated.Status) {
+	if !IngressStatusesEqual(current.Status, updated.Status) {
 		if err := r.client.Status().Update(context.TODO(), updated); err != nil {
 			return fmt.Errorf("failed to update status: %v", err)
 		}
@@ -418,7 +432,7 @@ func (r *reconciler) validate(ic *operatorv1.IngressController) error {
 	var errors []error
 
 	ingresses := &operatorv1.IngressControllerList{}
-	if err := r.cache.List(context.TODO(), ingresses, client.InNamespace(r.Namespace)); err != nil {
+	if err := r.cache.List(context.TODO(), ingresses, client.InNamespace(r.config.Namespace)); err != nil {
 		return fmt.Errorf("failed to list ingresscontrollers: %v", err)
 	}
 
@@ -669,13 +683,13 @@ func (r *reconciler) ensureIngressController(ci *operatorv1.IngressController, d
 	}
 
 	operandEvents := &corev1.EventList{}
-	if err := r.cache.List(context.TODO(), operandEvents, client.InNamespace("openshift-ingress")); err != nil {
-		errs = append(errs, fmt.Errorf("failed to list events in namespace %q: %v", "openshift-ingress", err))
+	if err := r.cache.List(context.TODO(), operandEvents, client.InNamespace(operatorcontroller.DefaultOperandNamespace)); err != nil {
+		errs = append(errs, fmt.Errorf("failed to list events in namespace %q: %v", operatorcontroller.DefaultOperandNamespace, err))
 	}
 
 	pods := &corev1.PodList{}
-	if err := r.cache.List(context.TODO(), pods, client.InNamespace("openshift-ingress")); err != nil {
-		errs = append(errs, fmt.Errorf("failed to list pods in namespace %q: %v", "openshift-ingress", err))
+	if err := r.cache.List(context.TODO(), pods, client.InNamespace(operatorcontroller.DefaultOperandNamespace)); err != nil {
+		errs = append(errs, fmt.Errorf("failed to list pods in namespace %q: %v", operatorcontroller.DefaultOperatorNamespace, err))
 	}
 
 	errs = append(errs, r.syncIngressControllerStatus(ci, deployment, pods.Items, lbService, operandEvents.Items, wildcardRecord, dnsConfig))
